@@ -1,13 +1,19 @@
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
- * Copyright (c) 2007-2008 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2007-2022 Cisco Systems, Inc.  All rights reserved
  * Copyright (c) 2004-2010 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2008-2009 Sun Microsystems, Inc.  All rights reserved.
+ * Copyright (c) 2015      Los Alamos National Security, LLC.  All rights
+ *                         reserved.
+ * Copyright (c) 2016      Intel, Inc. All rights reserved.
+ * Copyright (c) 2016      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 
@@ -58,9 +64,9 @@
  * Oct 27 1997 JHC: Created by exploding db_message_state_mpich.cxx
  */
 
-/* 
-   The following was added by William Gropp to improve the portability 
-   to systems with non-ANSI C compilers 
+/*
+   The following was added by William Gropp to improve the portability
+   to systems with non-ANSI C compilers
  */
 
 #include "ompi_config.h"
@@ -68,20 +74,16 @@
 #ifdef HAVE_NO_C_CONST
 #define const
 #endif
-#if defined(HAVE_STRING_H)
 #include <string.h>
-#endif  /* defined(HAVE_STRING_H) */
-#if defined(HAVE_STDLIB_H)
 #include <stdlib.h>
-#endif  /* defined(HAVE_STDLIB_H) */
 
 /* Notice to developers!!!!
- * The following include files with _dbg.h suffixes contains definitions 
- * that are shared between the debuggger plugins and the OMPI code base.
+ * The following include files with _dbg.h suffixes contains definitions
+ * that are shared between the debugger plugins and the OMPI code base.
  * This is done instead of including the non-_dbg suffixed files because
- * of the different way compilers may handle extern definitions. The 
- * particular case that is causing problems is when there is an extern 
- * variable or function that is accessed in a static inline function. 
+ * of the different way compilers may handle extern definitions. The
+ * particular case that is causing problems is when there is an extern
+ * variable or function that is accessed in a static inline function.
  * For example, here is the code we often see in a header file.
  *
  * extern int request_complete;
@@ -104,12 +106,12 @@
  *
  * If this code exists it actually compiles fine however an undefined symbol
  * is kept for foo() and in the case of some tools that load in plugins with
- * RTLD_NOW this undefined symbol causes the dlopen to fail since we do not 
+ * RTLD_NOW this undefined symbol causes the dlopen to fail since we do not
  * have (nor really need) the supporting library containing foo().
  *
- * Therefore, to handle cases like the above with compilers that require the 
- * symbols (like Sun Studio) instead of  pulling in all of OMPI into the 
- * plugins or defining dummy symbols here we separate the definitions used by 
+ * Therefore, to handle cases like the above with compilers that require the
+ * symbols (like Sun Studio) instead of  pulling in all of OMPI into the
+ * plugins or defining dummy symbols here we separate the definitions used by
  * both sets of code into the _dbg.h files.
  *
  * This means if one needs to add another definition that the plugins must see
@@ -124,7 +126,7 @@
 #include "msgq_interface.h"
 #include "ompi_msgq_dll_defs.h"
 
-/* 
+/*
    End of inclusion
  */
 
@@ -186,11 +188,16 @@ int mqs_version_compatibility (void)
     return MQS_INTERFACE_COMPATIBILITY;
 } /* mqs_version_compatibility */
 
+static char mqs_version_str[OMPI_MAX_VER_SIZE];
+
 /* This one can say what you like */
 char *mqs_version_string (void)
 {
-    return "Open MPI message queue support for parallel"
-           " debuggers compiled on " __DATE__;
+    int offset;
+    offset = snprintf(mqs_version_str, OMPI_MAX_VER_SIZE-1,  
+                      "Open MPI message queue support for parallel debuggers ");
+    ompi_get_lib_version(mqs_version_str+offset, OMPI_MAX_VER_SIZE-offset);
+    return mqs_version_str;
 } /* mqs_version_string */
 
 /* So the debugger can tell what interface width the library was compiled with */
@@ -206,12 +213,12 @@ int mqs_dll_taddr_width (void)
  */
 /**********************************************************************/
 /* Translate a process number */
-static int translate (group_t *this, int index) 
-{ 	
+static int translate (group_t *this, int index)
+{
     if (index == MQS_INVALID_PROCESS ||
         ((unsigned int)index) >= ((unsigned int) this->entries))
         return MQS_INVALID_PROCESS;
-    return this->local_to_global[index]; 
+    return this->local_to_global[index];
 } /* translate */
 
 /**********************************************************************/
@@ -239,7 +246,7 @@ static group_t * find_or_create_group( mqs_process *proc,
         DEBUG(VERBOSE_COMM, ("Get a size for the communicator = %d\n", np));
         return NULL;  /* Makes no sense ! */
     }
-    is_dense = 
+    is_dense =
         ompi_fetch_int( proc,
                         group_base + i_info->ompi_group_t.offset.grp_flags,
                         p_info );
@@ -256,7 +263,7 @@ static group_t * find_or_create_group( mqs_process *proc,
         }
     }
 
-    /* Hmm, couldn't find one, so fetch it */	
+    /* Hmm, couldn't find one, so fetch it */
     group = (group_t *)mqs_malloc (sizeof (group_t));
     tr = (int *)mqs_malloc (np*sizeof(int));
     trbuffer = (char *)mqs_malloc (np*sizeof(mqs_taddr_t));
@@ -264,7 +271,7 @@ static group_t * find_or_create_group( mqs_process *proc,
     group->group_base = group_base;
     DEBUG(VERBOSE_GROUP, ("Create a new group 0x%p with %d members\n",
                           (void*)group, np) );
-    
+
     tablep = ompi_fetch_pointer( proc,
                                  group_base + i_info->ompi_group_t.offset.grp_proc_pointers,
                                  p_info);
@@ -354,7 +361,7 @@ int mqs_setup_image (mqs_image *image, const mqs_image_callbacks *icb)
     i_info->extra = NULL;
 
     mqs_put_image_info (image, (mqs_image_info *)i_info);
-  
+
     return mqs_ok;
 } /* mqs_setup_image */
 
@@ -381,7 +388,7 @@ int mqs_image_has_queues (mqs_image *image, char **message)
      * that types have been read from there before we try to look them
      * up.
      */
-    mqs_find_function (image, "MPIR_Breakpoint", mqs_lang_c, NULL);
+    mqs_find_function (image, "ompi_debugger_setup_dlls", mqs_lang_c, NULL);
 
     /* Are we supposed to ignore this ? (e.g. it's really an HPF
      * runtime using the Open MPI process acquisition, but not wanting
@@ -403,7 +410,7 @@ int mqs_image_has_queues (mqs_image *image, char **message)
  * has no message queue information.
  */
 int mqs_setup_process (mqs_process *process, const mqs_process_callbacks *pcb)
-{ 
+{
     /* Extract the addresses of the global variables we need and save them away */
     mpi_process_info *p_info = (mpi_process_info *)mqs_malloc (sizeof (mpi_process_info));
 
@@ -442,46 +449,47 @@ int mqs_setup_process (mqs_process *process, const mqs_process_callbacks *pcb)
         {
             mqs_taddr_t typedefs_sizeof;
 
-            if(mqs_find_symbol (image, "MPIR_debug_typedefs_sizeof", &typedefs_sizeof) != mqs_ok)
-               return err_no_store;
-               p_info->sizes.short_size = ompi_fetch_int( process, /* sizeof (short) */
-                                                          typedefs_sizeof,
-                                                          p_info );
-               typedefs_sizeof += p_info->sizes.int_size;
-               p_info->sizes.int_size = ompi_fetch_int( process, /* sizeof (int) */
-                                                        typedefs_sizeof,
-                                                        p_info );
-               typedefs_sizeof += p_info->sizes.int_size;
-               p_info->sizes.long_size = ompi_fetch_int( process, /* sizeof (long) */
-                                                         typedefs_sizeof,
-                                                         p_info );
-               typedefs_sizeof += p_info->sizes.int_size;
-               p_info->sizes.long_long_size = ompi_fetch_int( process, /* sizeof (long long) */
-                                                              typedefs_sizeof,
-                                                              p_info );
-               typedefs_sizeof += p_info->sizes.int_size;
-               p_info->sizes.pointer_size = ompi_fetch_int( process, /* sizeof (void *) */
-                                                            typedefs_sizeof,
-                                                            p_info );
-               typedefs_sizeof += p_info->sizes.int_size;
-               p_info->sizes.bool_size = ompi_fetch_int( process, /* sizeof (bool) */
-                                                         typedefs_sizeof,
-                                                         p_info );
-               typedefs_sizeof += p_info->sizes.int_size;
-               p_info->sizes.size_t_size = ompi_fetch_int( process, /* sizeof (size_t) */
+            if (mqs_find_symbol (image, "MPIR_debug_typedefs_sizeof", &typedefs_sizeof) != mqs_ok) {
+                return err_no_store;
+            }
+            p_info->sizes.short_size = ompi_fetch_int( process, /* sizeof (short) */
+                                                       typedefs_sizeof,
+                                                       p_info );
+            typedefs_sizeof += p_info->sizes.int_size;
+            p_info->sizes.int_size = ompi_fetch_int( process, /* sizeof (int) */
+                                                     typedefs_sizeof,
+                                                     p_info );
+            typedefs_sizeof += p_info->sizes.int_size;
+             p_info->sizes.long_size = ompi_fetch_int( process, /* sizeof (long) */
+                                                       typedefs_sizeof,
+                                                       p_info );
+            typedefs_sizeof += p_info->sizes.int_size;
+            p_info->sizes.long_long_size = ompi_fetch_int( process, /* sizeof (long long) */
                                                            typedefs_sizeof,
                                                            p_info );
-               DEBUG( VERBOSE_GENERAL, 
-                      ("sizes short = %d int = %d long = %d long long = %d "
-                       "void* = %d bool = %d size_t = %d\n",
-                       p_info->sizes.short_size, p_info->sizes.int_size,
-                       p_info->sizes.long_size, p_info->sizes.long_long_size,
-                       p_info->sizes.pointer_size, p_info->sizes.bool_size,
-                       p_info->sizes.size_t_size) );
+            typedefs_sizeof += p_info->sizes.int_size;
+            p_info->sizes.pointer_size = ompi_fetch_int( process, /* sizeof (void *) */
+                                                         typedefs_sizeof,
+                                                         p_info );
+            typedefs_sizeof += p_info->sizes.int_size;
+            p_info->sizes.bool_size = ompi_fetch_int( process, /* sizeof (bool) */
+                                                      typedefs_sizeof,
+                                                      p_info );
+            typedefs_sizeof += p_info->sizes.int_size;
+            p_info->sizes.size_t_size = ompi_fetch_int( process, /* sizeof (size_t) */
+                                                        typedefs_sizeof,
+                                                        p_info );
+            DEBUG( VERBOSE_GENERAL,
+                   ("sizes short = %d int = %d long = %d long long = %d "
+                    "void* = %d bool = %d size_t = %d\n",
+                    p_info->sizes.short_size, p_info->sizes.int_size,
+                    p_info->sizes.long_size, p_info->sizes.long_long_size,
+                    p_info->sizes.pointer_size, p_info->sizes.bool_size,
+                    p_info->sizes.size_t_size) );
         }
 
         mqs_put_process_info (process, (mqs_process_info *)p_info);
-      
+
         return mqs_ok;
     }
     return err_no_store;
@@ -502,10 +510,10 @@ int mqs_process_has_queues (mqs_process *proc, char **msg)
     DEBUG(VERBOSE_GENERAL,("checking the status of the OMPI dll\n"));
     if (mqs_find_symbol (image, "ompi_mpi_communicators", &extra->commlist_base) != mqs_ok)
         return err_all_communicators;
-  
+
     if (mqs_find_symbol (image, "mca_pml_base_send_requests", &extra->send_queue_base) != mqs_ok)
         return err_mpid_sends;
-  
+
     if (mqs_find_symbol (image, "mca_pml_base_recv_requests", &extra->recv_queue_base) != mqs_ok)
         return err_mpid_recvs;
     DEBUG(VERBOSE_GENERAL,("process_has_queues returned success\n"));
@@ -513,7 +521,7 @@ int mqs_process_has_queues (mqs_process *proc, char **msg)
 } /* mqs_process_has_queues */
 
 /***********************************************************************
- * Check if the communicators have changed by looking at the 
+ * Check if the communicators have changed by looking at the
  * pointer array values for lowest_free and number_free.
  */
 static int communicators_changed (mqs_process *proc)
@@ -577,7 +585,7 @@ static int compare_comms (const void *a, const void *b)
 } /* compare_comms */
 
 /***********************************************************************
- * Rebuild our list of communicators because something has changed 
+ * Rebuild our list of communicators because something has changed
  */
 static int rebuild_communicator_list (mqs_process *proc)
 {
@@ -586,7 +594,7 @@ static int rebuild_communicator_list (mqs_process *proc)
     mqs_image * image        = mqs_get_image (proc);
     mpi_image_info *i_info   = (mpi_image_info *)mqs_get_image_info (image);
     communicator_t **commp, *old;
-    int i, commcount = 0, context_id, local_rank;
+    int i, commcount = 0, context_id;
     mqs_tword_t comm_size, lowest_free, number_free;
     mqs_taddr_t comm_addr_base;
     mqs_taddr_t comm_ptr;
@@ -634,7 +642,7 @@ static int rebuild_communicator_list (mqs_process *proc)
                         (long long)comm_addr_base, (int)sizeof(mqs_taddr_t)));
     for( i = 0; (commcount < (comm_size - number_free)) && (i < comm_size); i++ ) {
         /* Get the communicator pointer */
-        comm_ptr = 
+        comm_ptr =
             ompi_fetch_pointer( proc,
                                 comm_addr_base + i * p_info->sizes.pointer_size,
                                 p_info );
@@ -642,16 +650,14 @@ static int rebuild_communicator_list (mqs_process *proc)
         if( 0 == comm_ptr ) continue;
         commcount++;
         /* Now let's grab the data we want from inside */
+        /* NTH:  XXXXXXXXXXXXX FIXME!!!!!!!!!!!!!! c_index is local but MSGQ needs a global identifier
+         * that is sizeof (void *) or smaller. */
         DEBUG(VERBOSE_GENERAL, ("Retrieve context_id from 0x%llx and local_rank from 0x%llx\n",
-                                (long long)(comm_ptr + i_info->ompi_communicator_t.offset.c_contextid),
+                                (long long)(comm_ptr + i_info->ompi_communicator_t.offset.c_index),
                                 (long long)(comm_ptr + i_info->ompi_communicator_t.offset.c_my_rank)));
         context_id = ompi_fetch_int( proc,
-                                     comm_ptr + i_info->ompi_communicator_t.offset.c_contextid,
+                                     comm_ptr + i_info->ompi_communicator_t.offset.c_index,
                                      p_info );
-        local_rank = ompi_fetch_int( proc,
-                                     comm_ptr + i_info->ompi_communicator_t.offset.c_my_rank,
-                                     p_info );
-
         /* Do we already have this communicator ? */
         old = find_communicator(p_info, context_id);
         if( NULL == old ) {
@@ -663,7 +669,9 @@ static int rebuild_communicator_list (mqs_process *proc)
             extra->communicator_list = old;
             old->comm_ptr             = comm_ptr;
             old->comm_info.unique_id  = context_id;
-            old->comm_info.local_rank = local_rank;
+            old->comm_info.local_rank = ompi_fetch_int(proc,
+                                                       comm_ptr + i_info->ompi_communicator_t.offset.c_my_rank,
+                                                       p_info);
             old->group = NULL;
 
             DEBUG(VERBOSE_COMM,("Create new communicator 0x%lx with context_id %d and local_rank %d\n",
@@ -862,9 +870,9 @@ static int next_item_opal_list_t( mqs_process *proc, mpi_process_info *p_info,
 
 #if defined(CODE_NOT_USED)
 /**
- * Parsing the ompi_free_list lists.
+ * Parsing the opal_free_list lists.
  */
-static void ompi_free_list_t_dump_position( mqs_ompi_free_list_t_pos* position )
+static void opal_free_list_t_dump_position( mqs_opal_free_list_t_pos* position )
 {
     printf( "position->opal_list_t_pos.current_item = 0x%llx\n", (long long)position->opal_list_t_pos.current_item );
     printf( "position->opal_list_t_pos.list         = 0x%llx\n", (long long)position->opal_list_t_pos.list );
@@ -883,8 +891,8 @@ static void ompi_free_list_t_dump_position( mqs_ompi_free_list_t_pos* position )
 }
 #endif  /* CODE_NOT_USED */
 
-static int ompi_free_list_t_init_parser( mqs_process *proc, mpi_process_info *p_info,
-                                         mqs_ompi_free_list_t_pos* position, mqs_taddr_t free_list )
+static int opal_free_list_t_init_parser( mqs_process *proc, mpi_process_info *p_info,
+                                         mqs_opal_free_list_t_pos* position, mqs_taddr_t free_list )
 {
     mqs_image * image          = mqs_get_image (proc);
     mpi_image_info *i_info   = (mpi_image_info *)mqs_get_image_info (image);
@@ -893,22 +901,22 @@ static int ompi_free_list_t_init_parser( mqs_process *proc, mpi_process_info *p_
     position->free_list = free_list;
 
     position->fl_frag_size =
-        ompi_fetch_size_t( proc, position->free_list + i_info->ompi_free_list_t.offset.fl_frag_size,
+        ompi_fetch_size_t( proc, position->free_list + i_info->opal_free_list_t.offset.fl_frag_size,
                            p_info );
     position->fl_frag_alignment =
-        ompi_fetch_size_t( proc, position->free_list + i_info->ompi_free_list_t.offset.fl_frag_alignment,
+        ompi_fetch_size_t( proc, position->free_list + i_info->opal_free_list_t.offset.fl_frag_alignment,
                            p_info );
     position->fl_frag_class =
-        ompi_fetch_pointer( proc, position->free_list + i_info->ompi_free_list_t.offset.fl_frag_class,
+        ompi_fetch_pointer( proc, position->free_list + i_info->opal_free_list_t.offset.fl_frag_class,
                             p_info );
     position->fl_mpool =
-        ompi_fetch_pointer( proc, position->free_list + i_info->ompi_free_list_t.offset.fl_mpool,
+        ompi_fetch_pointer( proc, position->free_list + i_info->opal_free_list_t.offset.fl_mpool,
                             p_info );
     position->fl_num_per_alloc =
-        ompi_fetch_size_t( proc, position->free_list + i_info->ompi_free_list_t.offset.fl_num_per_alloc,
+        ompi_fetch_size_t( proc, position->free_list + i_info->opal_free_list_t.offset.fl_num_per_alloc,
                            p_info );
     position->fl_num_allocated =
-        ompi_fetch_size_t( proc, position->free_list + i_info->ompi_free_list_t.offset.fl_num_allocated,
+        ompi_fetch_size_t( proc, position->free_list + i_info->opal_free_list_t.offset.fl_num_allocated,
                            p_info );
 
     if( 0 == position->fl_mpool ) {
@@ -921,7 +929,7 @@ static int ompi_free_list_t_init_parser( mqs_process *proc, mpi_process_info *p_
                                          position->fl_frag_alignment, mqs_taddr_t );
 
     /**
-     * Work around the strange ompi_free_list_t way to allocate elements. The first chunk is
+     * Work around the strange opal_free_list_t way to allocate elements. The first chunk is
      * not required to have the same size as the others.
      * A similar work around should be set for the last chunk of allocations too !!! But how
      * can we solve ONE equation with 2 unknowns ?
@@ -933,7 +941,7 @@ static int ompi_free_list_t_init_parser( mqs_process *proc, mpi_process_info *p_
         if( 0 == position->fl_num_initial_alloc )
             position->fl_num_initial_alloc = position->fl_num_per_alloc;
     }
-    DEBUG(VERBOSE_LISTS,("ompi_free_list_t fl_frag_size = %lld fl_header_space = %lld\n"
+    DEBUG(VERBOSE_LISTS,("opal_free_list_t fl_frag_size = %lld fl_header_space = %lld\n"
                          "                 fl_frag_alignment = %lld fl_num_per_alloc = %lld\n"
                          "                 fl_num_allocated = %lld fl_num_initial_alloc = %lld\n"
                          "                 header_space = %lld\n",
@@ -946,7 +954,7 @@ static int ompi_free_list_t_init_parser( mqs_process *proc, mpi_process_info *p_
      * Initialize the pointer to the opal_list_t.
      */
     opal_list_t_init_parser( proc, p_info, &position->opal_list_t_pos,
-                             position->free_list + i_info->ompi_free_list_t.offset.fl_allocations );
+                             position->free_list + i_info->opal_free_list_t.offset.fl_allocations );
     next_item_opal_list_t( proc, p_info, &position->opal_list_t_pos, &active_allocation );
     DEBUG(VERBOSE_LISTS,("active_allocation 0x%llx header_space %d\n",
                          (long long)active_allocation, (int)position->header_space));
@@ -956,7 +964,7 @@ static int ompi_free_list_t_init_parser( mqs_process *proc, mpi_process_info *p_
         /**
          * Handle alignment issues...
          */
-        active_allocation += i_info->ompi_free_list_item_t.size;
+        active_allocation += i_info->opal_free_list_item_t.size;
         active_allocation = OPAL_ALIGN( active_allocation,
                                         position->fl_frag_alignment, mqs_taddr_t );
         /**
@@ -969,16 +977,16 @@ static int ompi_free_list_t_init_parser( mqs_process *proc, mpi_process_info *p_
                              (long long)active_allocation, (long long)position->upper_bound));
     }
     position->current_item = active_allocation;
-    
-    /*ompi_free_list_t_dump_position( position );*/
+
+    /*opal_free_list_t_dump_position( position );*/
     return mqs_ok;
 }
 
 /**
  * Return the current position and move the internal counter to the next element.
  */
-static int ompi_free_list_t_next_item( mqs_process *proc, mpi_process_info *p_info,
-                                       mqs_ompi_free_list_t_pos* position, mqs_taddr_t* active_item )
+static int opal_free_list_t_next_item( mqs_process *proc, mpi_process_info *p_info,
+                                       mqs_opal_free_list_t_pos* position, mqs_taddr_t* active_item )
 {
     mqs_image * image          = mqs_get_image (proc);
     mpi_image_info *i_info   = (mpi_image_info *)mqs_get_image_info (image);
@@ -990,7 +998,7 @@ static int ompi_free_list_t_next_item( mqs_process *proc, mpi_process_info *p_in
 
     position->current_item += position->header_space;
     if( position->current_item >= position->upper_bound ) {
-        DEBUG(VERBOSE_LISTS,("Reach the end of one of the ompi_free_list_t "
+        DEBUG(VERBOSE_LISTS,("Reach the end of one of the opal_free_list_t "
                              "allocations. Go to the next one\n"));
         /* we should go to the next allocation */
         next_item_opal_list_t( proc, p_info,
@@ -1002,7 +1010,7 @@ static int ompi_free_list_t_next_item( mqs_process *proc, mpi_process_info *p_in
         /**
          * Handle alignment issues...
          */
-        active_allocation += i_info->ompi_free_list_item_t.size;
+        active_allocation += i_info->opal_free_list_item_t.size;
         active_allocation = OPAL_ALIGN( active_allocation,
                                         position->fl_frag_alignment, mqs_taddr_t );
         /**
@@ -1014,7 +1022,7 @@ static int ompi_free_list_t_next_item( mqs_process *proc, mpi_process_info *p_in
         DEBUG(VERBOSE_LISTS,("there are more elements in the list "
                              "active_allocation = %llx upper_bound = %llx\n",
                              (long long)active_allocation, (long long)position->upper_bound));
-        /*ompi_free_list_t_dump_position( position );*/
+        /*opal_free_list_t_dump_position( position );*/
     }
     DEBUG(VERBOSE_LISTS,("Free list actual position 0x%llx next element at 0x%llx\n",
                          (long long)*active_item, (long long)position->current_item));
@@ -1060,10 +1068,6 @@ static void dump_request( mqs_taddr_t current_item, mqs_pending_operation *res )
     printf( "+===============================================+\n\n" );
 }
 
-/**
- * TODO: ompi_request_completed can be used to detect any changes in the request handles.
- */
-
 /**
  * Handle the send queue as well as the receive queue. The unexpected queue
  * is a whole different story ...
@@ -1081,7 +1085,7 @@ static int fetch_request( mqs_process *proc, mpi_process_info *p_info,
     /* If we get a PML request with an internal tag we will jump back here */
   rescan_requests:
     while( 1 ) {
-        ompi_free_list_t_next_item( proc, p_info,
+        opal_free_list_t_next_item( proc, p_info,
                                     &extra->next_msg, &current_item );
         if( 0 == current_item ) {
             DEBUG(VERBOSE_REQ,("no more items in the %s request queue\n",
@@ -1136,12 +1140,12 @@ static int fetch_request( mqs_process *proc, mpi_process_info *p_info,
         res->desired_local_rank  = ompi_fetch_int( proc, current_item + i_info->mca_pml_base_request_t.offset.req_peer, p_info );
         res->desired_global_rank = translate( extra->current_communicator->group,
                                               res->desired_local_rank );
-        
+
         res->buffer = ompi_fetch_pointer( proc, current_item + i_info->mca_pml_base_request_t.offset.req_addr,
                                      p_info );
         /* Set this to true if it's a buffered request */
         res->system_buffer = FALSE;
-            
+
         /* The pointer to the request datatype */
         ompi_datatype =
             ompi_fetch_pointer( proc,
@@ -1155,8 +1159,18 @@ static int fetch_request( mqs_process *proc, mpi_process_info *p_info,
         mqs_fetch_data( proc, ompi_datatype + i_info->ompi_datatype_t.offset.name,
                         64, data_name );
         if( '\0' != data_name[0] ) {
-            snprintf( (char*)res->extra_text[1], 64, "Data: %d * %s",
-                      (int)res->desired_length, data_name );
+            // res->extra_text[x] is only 64 chars long -- same as
+            // data_name.  If you try to snprintf it into
+            // res->extra_text with additional text, some compilers
+            // will warn that we might truncate the string (because it
+            // can see the static char array lengths).  So just put
+            // data_name in res->extra_text[2] (vs. extra_text[1]),
+            // where it is guaranteed to fit.
+            data_name[4] = '\0';
+            snprintf( (char*)res->extra_text[1], 64, "Data: %d instances of MPI datatype",
+                      (int)res->desired_length);
+            snprintf( (char*)res->extra_text[2], 64, "%s",
+                      data_name );
         }
         /* And now compute the real length as specified by the user */
         res->desired_length *=
@@ -1209,7 +1223,7 @@ static int fetch_request( mqs_process *proc, mpi_process_info *p_info,
         /* If the length we're looking for is the count ... */
         /*res->desired_length      =
           ompi_fetch_int( proc, current_item + i_info->mca_pml_base_request_t.offset.req_count, p_info );*/
-        
+
         if( (mqs_st_pending < res->status) && (MCA_PML_REQUEST_SEND != req_type) ) {  /* The real data from the status */
             res->actual_length       =
                 ompi_fetch_size_t( proc, current_item + i_info->ompi_request_t.offset.req_status +
@@ -1229,7 +1243,7 @@ static int fetch_request( mqs_process *proc, mpi_process_info *p_info,
 }
 
 /***********************************************************************
- * Setup to iterate over pending operations 
+ * Setup to iterate over pending operations
  */
 int mqs_setup_operation_iterator (mqs_process *proc, int op)
 {
@@ -1241,12 +1255,12 @@ int mqs_setup_operation_iterator (mqs_process *proc, int op)
     switch (op) {
     case mqs_pending_sends:
         DEBUG(VERBOSE_REQ,("setup the send queue iterator\n"));
-        ompi_free_list_t_init_parser( proc, p_info, &extra->next_msg, extra->send_queue_base );
+        opal_free_list_t_init_parser( proc, p_info, &extra->next_msg, extra->send_queue_base );
         return mqs_ok;
 
     case mqs_pending_receives:
         DEBUG(VERBOSE_REQ,("setup the receive queue iterator\n"));
-        ompi_free_list_t_init_parser( proc, p_info, &extra->next_msg, extra->recv_queue_base );
+        opal_free_list_t_init_parser( proc, p_info, &extra->next_msg, extra->recv_queue_base );
         return mqs_ok;
 
     case mqs_unexpected_messages:  /* TODO */
@@ -1258,7 +1272,7 @@ int mqs_setup_operation_iterator (mqs_process *proc, int op)
 } /* mqs_setup_operation_iterator */
 
 /***********************************************************************
- * Fetch the next valid operation. 
+ * Fetch the next valid operation.
  * Since Open MPI only maintains a single queue of each type of operation,
  * we have to run over it and filter out the operations which
  * match the active communicator.
@@ -1296,11 +1310,11 @@ void mqs_destroy_process_info (mqs_process_info *mp_info)
         comm = extra->communicator_list;
         while (comm) {
             communicator_t *next = comm->next;
-            
+
             if( NULL != comm->group )
                 group_decref (comm->group);  /* Group is no longer referenced from here */
             mqs_free (comm);
-            
+
             comm = next;
         }
         if (NULL != extra) {
@@ -1326,95 +1340,95 @@ char * mqs_dll_error_string (int errcode)
     switch (errcode) {
     case err_silent_failure:
         return "";
-    case err_no_current_communicator: 
+    case err_no_current_communicator:
         return "No current communicator in the communicator iterator";
-    case err_bad_request:    
+    case err_bad_request:
         return "Attempting to setup to iterate over an unknown queue of operations";
-    case err_no_store: 
+    case err_no_store:
         return "Unable to allocate store";
-    case err_failed_qhdr: 
+    case err_failed_qhdr:
         return "Failed to find type MPID_QHDR";
-    case err_unexpected: 
+    case err_unexpected:
         return "Failed to find field 'unexpected' in MPID_QHDR";
-    case err_posted: 
+    case err_posted:
         return "Failed to find field 'posted' in MPID_QHDR";
-    case err_failed_queue: 
+    case err_failed_queue:
         return "Failed to find type MPID_QUEUE";
-    case err_first: 
+    case err_first:
         return "Failed to find field 'first' in MPID_QUEUE";
-    case err_context_id: 
+    case err_context_id:
         return "Failed to find field 'context_id' in MPID_QEL";
-    case err_tag: 
+    case err_tag:
         return "Failed to find field 'tag' in MPID_QEL";
-    case err_tagmask: 
+    case err_tagmask:
         return "Failed to find field 'tagmask' in MPID_QEL";
-    case err_lsrc: 
+    case err_lsrc:
         return "Failed to find field 'lsrc' in MPID_QEL";
-    case err_srcmask: 
+    case err_srcmask:
         return "Failed to find field 'srcmask' in MPID_QEL";
-    case err_next: 
+    case err_next:
         return "Failed to find field 'next' in MPID_QEL";
-    case err_ptr: 
+    case err_ptr:
         return "Failed to find field 'ptr' in MPID_QEL";
-    case err_missing_type: 
+    case err_missing_type:
         return "Failed to find some type";
-    case err_missing_symbol: 
+    case err_missing_symbol:
         return "Failed to find field the global symbol";
-    case err_db_shandle: 
+    case err_db_shandle:
         return "Failed to find field 'db_shandle' in MPIR_SQEL";
-    case err_db_comm: 
+    case err_db_comm:
         return "Failed to find field 'db_comm' in MPIR_SQEL";
-    case err_db_target: 
+    case err_db_target:
         return "Failed to find field 'db_target' in MPIR_SQEL";
-    case err_db_tag: 
+    case err_db_tag:
         return "Failed to find field 'db_tag' in MPIR_SQEL";
-    case err_db_data: 
+    case err_db_data:
         return "Failed to find field 'db_data' in MPIR_SQEL";
-    case err_db_byte_length: 
+    case err_db_byte_length:
         return "Failed to find field 'db_byte_length' in MPIR_SQEL";
-    case err_db_next: 
+    case err_db_next:
         return "Failed to find field 'db_next' in MPIR_SQEL";
-    case err_failed_rhandle: 
+    case err_failed_rhandle:
         return "Failed to find type MPIR_RHANDLE";
-    case err_is_complete: 
+    case err_is_complete:
         return "Failed to find field 'is_complete' in MPIR_RHANDLE";
-    case err_buf: 
+    case err_buf:
         return "Failed to find field 'buf' in MPIR_RHANDLE";
-    case err_len: 
+    case err_len:
         return "Failed to find field 'len' in MPIR_RHANDLE";
-    case err_s: 
+    case err_s:
         return "Failed to find field 's' in MPIR_RHANDLE";
-    case err_failed_status: 
+    case err_failed_status:
         return "Failed to find type MPI_Status";
-    case err_count: 
+    case err_count:
         return "Failed to find field 'count' in MPIR_Status";
-    case err_MPI_SOURCE: 
+    case err_MPI_SOURCE:
         return "Failed to find field 'MPI_SOURCE' in MPIR_Status";
-    case err_MPI_TAG: 
+    case err_MPI_TAG:
         return "Failed to find field 'MPI_TAG' in MPIR_Status";
-    case err_failed_commlist: 
+    case err_failed_commlist:
         return "Failed to find type MPIR_Comm_list";
-    case err_sequence_number: 
+    case err_sequence_number:
         return "Failed to find field 'sequence_number' in MPIR_Comm_list";
-    case err_comm_first: 
+    case err_comm_first:
         return "Failed to find field 'comm_first' in MPIR_Comm_list";
-    case err_failed_communicator: 
+    case err_failed_communicator:
         return "Failed to find type MPIR_Communicator";
-    case err_lrank_to_grank: 
+    case err_lrank_to_grank:
         return "Failed to find field 'lrank_to_grank' in MPIR_Communicator";
-    case err_send_context: 
+    case err_send_context:
         return "Failed to find field 'send_context' in MPIR_Communicator";
-    case err_recv_context: 
+    case err_recv_context:
         return "Failed to find field 'recv_context' in MPIR_Communicator";
-    case err_comm_next: 
+    case err_comm_next:
         return "Failed to find field 'comm_next' in MPIR_Communicator";
-    case err_comm_name: 
+    case err_comm_name:
         return "Failed to find field 'comm_name' in MPIR_Communicator";
-    case err_all_communicators: 
+    case err_all_communicators:
         return "Failed to find the global symbol MPIR_All_communicators";
-    case err_mpid_sends: 
+    case err_mpid_sends:
         return "Failed to access the global send requests list";
-    case err_mpid_recvs: 
+    case err_mpid_recvs:
         return "Failed to access the global receive requests list";
     case err_group_corrupt:
         return "Could not read a communicator's group from the process (probably a store corruption)";
