@@ -11,7 +11,7 @@
  *                         All rights reserved.
  * Copyright (c) 2006-2007 University of Houston. All rights reserved.
  * Copyright (c) 2013      Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2015-2016 Research Organization for Information Science
+ * Copyright (c) 2015-2017 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
  *
@@ -27,10 +27,10 @@
 #include "ompi/constants.h"
 #include "ompi/datatype/ompi_datatype.h"
 #include "ompi/communicator/communicator.h"
-#include "ompi/request/request.h"
 #include "ompi/op/op.h"
 #include "ompi/mca/coll/coll.h"
 #include "ompi/mca/coll/base/coll_tags.h"
+#include "ompi/mca/coll/base/coll_base_util.h"
 #include "ompi/mca/pml/pml.h"
 
 /*
@@ -48,8 +48,7 @@ mca_coll_inter_allreduce_inter(const void *sbuf, void *rbuf, int count,
                                mca_coll_base_module_t *module)
 {
     int err, rank, root = 0;
-    char *tmpbuf = NULL, *pml_buffer = NULL;
-    ompi_request_t *req[2];
+    char *tmpbuf = NULL, *pml_buffer = NULL, *source;
     ptrdiff_t gap, span;
 
     rank = ompi_comm_rank(comm);
@@ -59,36 +58,26 @@ mca_coll_inter_allreduce_inter(const void *sbuf, void *rbuf, int count,
 
     tmpbuf = (char *) malloc(span);
     if (NULL == tmpbuf) {
-	return OMPI_ERR_OUT_OF_RESOURCE;
+        return OMPI_ERR_OUT_OF_RESOURCE;
     }
     pml_buffer = tmpbuf - gap;
+    source = (MPI_IN_PLACE == sbuf) ? rbuf : sbuf;
 
-    err = comm->c_local_comm->c_coll->coll_reduce(sbuf, pml_buffer, count,
-						 dtype, op, root,
-						 comm->c_local_comm,
-                                                 comm->c_local_comm->c_coll->coll_reduce_module);
+    err = comm->c_local_comm->c_coll->coll_reduce(source, pml_buffer, count,
+                                                  dtype, op, root,
+                                                  comm->c_local_comm,
+                                                  comm->c_local_comm->c_coll->coll_reduce_module);
     if (OMPI_SUCCESS != err) {
-	goto exit;
+        goto exit;
     }
 
     if (rank == root) {
-	/* Do a send-recv between the two root procs. to avoid deadlock */
-        err = MCA_PML_CALL(irecv(rbuf, count, dtype, 0,
-                                 MCA_COLL_BASE_TAG_ALLREDUCE, comm,
-                                 &(req[0])));
-        if (OMPI_SUCCESS != err) {
-            goto exit;
-        }
-
-        err = MCA_PML_CALL(isend(pml_buffer, count, dtype, 0,
-                                 MCA_COLL_BASE_TAG_ALLREDUCE,
-                                 MCA_PML_BASE_SEND_STANDARD,
-                                 comm, &(req[1])));
-        if (OMPI_SUCCESS != err) {
-            goto exit;
-        }
-
-        err = ompi_request_wait_all(2, req, MPI_STATUSES_IGNORE);
+        /* Do a send-recv between the two root procs. to avoid deadlock */
+        err = ompi_coll_base_sendrecv_actual(pml_buffer, count, dtype, 0,
+                                             MCA_COLL_BASE_TAG_ALLREDUCE,
+                                             rbuf, count, dtype, 0,
+                                             MCA_COLL_BASE_TAG_ALLREDUCE,
+                                             comm, MPI_STATUS_IGNORE);
         if (OMPI_SUCCESS != err) {
             goto exit;
         }
@@ -96,8 +85,8 @@ mca_coll_inter_allreduce_inter(const void *sbuf, void *rbuf, int count,
 
     /* bcast the message to all the local processes */
     err = comm->c_local_comm->c_coll->coll_bcast(rbuf, count, dtype,
-						root, comm->c_local_comm,
-                                                comm->c_local_comm->c_coll->coll_bcast_module);
+                                                 root, comm->c_local_comm,
+                                                 comm->c_local_comm->c_coll->coll_bcast_module);
     if (OMPI_SUCCESS != err) {
             goto exit;
     }
