@@ -5,11 +5,13 @@
  *                         Corporation.  All rights reserved.
  * Copyright (c) 2006      The Technical University of Chemnitz. All
  *                         rights reserved.
- * Copyright (c) 2014-2017 Research Organization for Information Science
- *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2014-2018 Research Organization for Information Science
+ *                         and Technology (RIST).  All rights reserved.
  * Copyright (c) 2015-2017 Los Alamos National Security, LLC. All rights
  *                         reserved.
  * Copyright (c) 2017      IBM Corporation.  All rights reserved.
+ * Copyright (c) 2018      FUJITSU LIMITED.  All rights reserved.
+ * Copyright (c) 2021      Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -36,21 +38,21 @@ static inline int a2av_sched_inplace(int rank, int p, NBC_Schedule *schedule,
                                     MPI_Aint ext, MPI_Datatype type, ptrdiff_t gap);
 
 /* an alltoallv schedule can not be cached easily because the contents
- * ot the recvcounts array may change, so a comparison of the address
+ * of the recvcounts array may change, so a comparison of the address
  * would not be sufficient ... we simply do not cache it */
 
 /* simple linear Alltoallv */
-int ompi_coll_libnbc_ialltoallv(const void* sendbuf, const int *sendcounts, const int *sdispls,
-                                MPI_Datatype sendtype, void* recvbuf, const int *recvcounts, const int *rdispls,
-                                MPI_Datatype recvtype, struct ompi_communicator_t *comm, ompi_request_t ** request,
-                                struct mca_coll_base_module_2_2_0_t *module)
+static int nbc_alltoallv_init(const void* sendbuf, const int *sendcounts, const int *sdispls,
+                              MPI_Datatype sendtype, void* recvbuf, const int *recvcounts, const int *rdispls,
+                              MPI_Datatype recvtype, struct ompi_communicator_t *comm, ompi_request_t ** request,
+                              mca_coll_base_module_t *module, bool persistent)
 {
   int rank, p, res;
   MPI_Aint sndext, rcvext;
   NBC_Schedule *schedule;
   char *rbuf, *sbuf, inplace;
-  ptrdiff_t gap, span;
-  NBC_Handle *handle;
+  ptrdiff_t gap = 0, span;
+  void * tmpbuf = NULL;
   ompi_coll_libnbc_module_t *libnbc_module = (ompi_coll_libnbc_module_t*) module;
 
   NBC_IN_PLACE(sendbuf, recvbuf, inplace);
@@ -64,11 +66,6 @@ int ompi_coll_libnbc_ialltoallv(const void* sendbuf, const int *sendcounts, cons
     return res;
   }
 
-  res = NBC_Init_handle (comm, &handle, libnbc_module);
-  if (OPAL_UNLIKELY(OMPI_SUCCESS != res)) {
-    return res;
-  }
-
   /* copy data to receivbuffer */
   if (inplace) {
     int count = 0;
@@ -78,42 +75,49 @@ int ompi_coll_libnbc_ialltoallv(const void* sendbuf, const int *sendcounts, cons
       }
     }
     span = opal_datatype_span(&recvtype->super, count, &gap);
+    /**
+     * If this process has no data to send or receive it can bail out early,
+     * but it needs to increase the nonblocking tag to stay in sync with the
+     * rest of the processes.
+     */
     if (OPAL_UNLIKELY(0 == span)) {
-      *request = &ompi_request_empty;
-      NBC_Return_handle (handle);
-      return MPI_SUCCESS;
+      ompi_coll_base_nbc_reserve_tags(comm, 1);
+      return nbc_get_noop_request(persistent, request);
     }
-    handle->tmpbuf = malloc(span);
-    if (OPAL_UNLIKELY(NULL == handle->tmpbuf)) {
-      NBC_Return_handle (handle);
+    tmpbuf = malloc(span);
+    if (OPAL_UNLIKELY(NULL == tmpbuf)) {
       return OMPI_ERR_OUT_OF_RESOURCE;
     }
+
     sendcounts = recvcounts;
     sdispls = rdispls;
+    sndext = rcvext;
   } else {
     res = ompi_datatype_type_extent (sendtype, &sndext);
     if (MPI_SUCCESS != res) {
       NBC_Error("MPI Error in ompi_datatype_type_extent() (%i)", res);
-      NBC_Return_handle (handle);
       return res;
     }
-    if (sendcounts[rank] != 0) {
-      rbuf = (char *) recvbuf + rdispls[rank] * rcvext;
-      sbuf = (char *) sendbuf + sdispls[rank] * sndext;
-      res = NBC_Copy (sbuf, sendcounts[rank], sendtype, rbuf, recvcounts[rank], recvtype, comm);
-      if (OPAL_UNLIKELY(OMPI_SUCCESS != res)) {
-        return res;
-      }
-    }
   }
 
   schedule = OBJ_NEW(NBC_Schedule);
   if (OPAL_UNLIKELY(NULL == schedule)) {
-    NBC_Return_handle (handle);
+    free(tmpbuf);
     return OMPI_ERR_OUT_OF_RESOURCE;
   }
 
 
+  if (!inplace && sendcounts[rank] != 0) {
+    rbuf = (char *) recvbuf + rdispls[rank] * rcvext;
+    sbuf = (char *) sendbuf + sdispls[rank] * sndext;
+    res = NBC_Sched_copy (sbuf, false, sendcounts[rank], sendtype,
+                          rbuf, false, recvcounts[rank], recvtype, schedule, false);
+    if (OPAL_UNLIKELY(OMPI_SUCCESS != res)) {
+      OBJ_RELEASE(schedule);
+      return res;
+    }
+  }
+
   if (inplace) {
     res = a2av_sched_inplace(rank, p, schedule, recvbuf, recvcounts,
                                  rdispls, rcvext, recvtype, gap);
@@ -123,40 +127,58 @@ int ompi_coll_libnbc_ialltoallv(const void* sendbuf, const int *sendcounts, cons
                             recvbuf, recvcounts, rdispls, rcvext, recvtype);
   }
   if (OPAL_UNLIKELY(OMPI_SUCCESS != res)) {
-    NBC_Return_handle (handle);
     OBJ_RELEASE(schedule);
+    free(tmpbuf);
     return res;
   }
 
   res = NBC_Sched_commit (schedule);
   if (OPAL_UNLIKELY(OMPI_SUCCESS != res)) {
-    NBC_Return_handle (handle);
     OBJ_RELEASE(schedule);
+    free(tmpbuf);
     return res;
   }
 
-  res = NBC_Start(handle, schedule);
+  res = NBC_Schedule_request(schedule, comm, libnbc_module, persistent, request, tmpbuf);
   if (OPAL_UNLIKELY(OMPI_SUCCESS != res)) {
-    NBC_Return_handle (handle);
     OBJ_RELEASE(schedule);
+    free(tmpbuf);
     return res;
   }
 
-  *request = (ompi_request_t *) handle;
-
   return OMPI_SUCCESS;
 }
 
+int ompi_coll_libnbc_ialltoallv(const void* sendbuf, const int *sendcounts, const int *sdispls,
+                                MPI_Datatype sendtype, void* recvbuf, const int *recvcounts, const int *rdispls,
+                                MPI_Datatype recvtype, struct ompi_communicator_t *comm, ompi_request_t ** request,
+                                mca_coll_base_module_t *module) {
+    int res = nbc_alltoallv_init(sendbuf, sendcounts, sdispls, sendtype,
+                                 recvbuf, recvcounts, rdispls, recvtype,
+                                 comm, request, module, false);
+    if (OPAL_UNLIKELY(OMPI_SUCCESS != res)) {
+        return res;
+    }
+  
+    res = NBC_Start(*(ompi_coll_libnbc_request_t **)request);
+    if (OPAL_UNLIKELY(OMPI_SUCCESS != res)) {
+        NBC_Return_handle (*(ompi_coll_libnbc_request_t **)request);
+        *request = &ompi_request_null.request;
+        return res;
+    }
+
+    return OMPI_SUCCESS;
+}
+
 /* simple linear Alltoallv */
-int ompi_coll_libnbc_ialltoallv_inter (const void* sendbuf, const int *sendcounts, const int *sdispls,
-				       MPI_Datatype sendtype, void* recvbuf, const int *recvcounts, const int *rdispls,
-				       MPI_Datatype recvtype, struct ompi_communicator_t *comm, ompi_request_t ** request,
-				       struct mca_coll_base_module_2_2_0_t *module)
+static int nbc_alltoallv_inter_init (const void* sendbuf, const int *sendcounts, const int *sdispls,
+                                     MPI_Datatype sendtype, void* recvbuf, const int *recvcounts, const int *rdispls,
+                                     MPI_Datatype recvtype, struct ompi_communicator_t *comm, ompi_request_t ** request,
+                                     mca_coll_base_module_t *module, bool persistent)
 {
   int res, rsize;
   MPI_Aint sndext, rcvext;
   NBC_Schedule *schedule;
-  NBC_Handle *handle;
   ompi_coll_libnbc_module_t *libnbc_module = (ompi_coll_libnbc_module_t*) module;
 
 
@@ -206,24 +228,36 @@ int ompi_coll_libnbc_ialltoallv_inter (const void* sendbuf, const int *sendcount
     return res;
   }
 
-  res = NBC_Init_handle(comm, &handle, libnbc_module);
+  res = NBC_Schedule_request(schedule, comm, libnbc_module, persistent, request, NULL);
   if (OPAL_UNLIKELY(OMPI_SUCCESS != res)) {
     OBJ_RELEASE(schedule);
     return res;
   }
 
-  res = NBC_Start(handle, schedule);
-  if (OPAL_UNLIKELY(OMPI_SUCCESS != res)) {
-    NBC_Return_handle (handle);
-    OBJ_RELEASE(schedule);
-    return res;
-  }
-
-  *request = (ompi_request_t *) handle;
-
   return OMPI_SUCCESS;
 }
 
+int ompi_coll_libnbc_ialltoallv_inter (const void* sendbuf, const int *sendcounts, const int *sdispls,
+				       MPI_Datatype sendtype, void* recvbuf, const int *recvcounts, const int *rdispls,
+				       MPI_Datatype recvtype, struct ompi_communicator_t *comm, ompi_request_t ** request,
+				       mca_coll_base_module_t *module) {
+    int res = nbc_alltoallv_inter_init(sendbuf, sendcounts, sdispls, sendtype,
+                                       recvbuf, recvcounts, rdispls, recvtype,
+                                       comm, request, module, false);
+    if (OPAL_UNLIKELY(OMPI_SUCCESS != res)) {
+        return res;
+    }
+  
+    res = NBC_Start(*(ompi_coll_libnbc_request_t **)request);
+    if (OPAL_UNLIKELY(OMPI_SUCCESS != res)) {
+        NBC_Return_handle (*(ompi_coll_libnbc_request_t **)request);
+        *request = &ompi_request_null.request;
+        return res;
+    }
+
+    return OMPI_SUCCESS;
+}
+
 __opal_attribute_unused__
 static inline int a2av_sched_linear(int rank, int p, NBC_Schedule *schedule,
                                     const void *sendbuf, const int *sendcounts, const int *sdispls,
@@ -362,3 +396,29 @@ static inline int a2av_sched_inplace(int rank, int p, NBC_Schedule *schedule,
 
   return OMPI_SUCCESS;
 }
+
+int ompi_coll_libnbc_alltoallv_init(const void* sendbuf, const int *sendcounts, const int *sdispls,
+                                    MPI_Datatype sendtype, void* recvbuf, const int *recvcounts, const int *rdispls,
+                                    MPI_Datatype recvtype, struct ompi_communicator_t *comm, MPI_Info info, ompi_request_t ** request,
+                                    mca_coll_base_module_t *module) {
+    int res = nbc_alltoallv_init(sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype,
+                                 comm, request, module, true);
+    if (OPAL_UNLIKELY(OMPI_SUCCESS != res)) {
+        return res;
+    }
+
+    return OMPI_SUCCESS;
+}
+
+int ompi_coll_libnbc_alltoallv_inter_init(const void* sendbuf, const int *sendcounts, const int *sdispls,
+                                          MPI_Datatype sendtype, void* recvbuf, const int *recvcounts, const int *rdispls,
+                                          MPI_Datatype recvtype, struct ompi_communicator_t *comm, MPI_Info info, ompi_request_t ** request,
+                                          mca_coll_base_module_t *module) {
+    int res = nbc_alltoallv_inter_init(sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype,
+                                       comm, request, module, true);
+    if (OPAL_UNLIKELY(OMPI_SUCCESS != res)) {
+        return res;
+    }
+
+    return OMPI_SUCCESS;
+}
