@@ -3,18 +3,22 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2016 The University of Tennessee and The University
+ * Copyright (c) 2004-2021 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2006-2012 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2006-2017 Cisco Systems, Inc.  All rights reserved
  * Copyright (c) 2009-2012 Oracle and/or its affiliates.  All rights reserved.
  * Copyright (c) 2012      Oak Ridge National Labs.  All rights reserved.
- * Copyright (c) 2015-2016 Los Alamos National Security, LLC.  All rights
+ * Copyright (c) 2015-2017 Los Alamos National Security, LLC.  All rights
  *                         reserved.
+ * Copyright (c) 2018      FUJITSU LIMITED.  All rights reserved.
+ * Copyright (c) 2018      Triad National Security, LLC. All rights
+ *                         reserved.
+ * Copyright (c) 2022      IBM Corporation.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -34,9 +38,10 @@
 #include "mpi.h"
 #include "opal/class/opal_free_list.h"
 #include "opal/class/opal_pointer_array.h"
-#include "opal/threads/condition.h"
-#include "opal/threads/wait_sync.h"
+#include "opal/mca/threads/condition.h"
+#include "opal/mca/threads/wait_sync.h"
 #include "ompi/constants.h"
+#include "ompi/runtime/params.h"
 
 BEGIN_C_DECLS
 
@@ -55,6 +60,26 @@ OMPI_DECLSPEC OBJ_CLASS_DECLARATION(ompi_request_t);
 
 struct ompi_request_t;
 
+/**
+ * Initiate one or more persistent requests.
+ *
+ * This function is called by MPI_START and MPI_STARTALL.
+ *
+ * When called by MPI_START, count is 1.
+ *
+ * When called by MPI_STARTALL, multiple requests which have the same
+ * req_start value are passed. This may help scheduling optimization
+ * of multiple communications.
+ *
+ * @param count (IN)        Number of requests
+ * @param requests (IN/OUT) Array of persistent requests
+ * @return                  OMPI_SUCCESS or failure status.
+ */
+typedef int (*ompi_request_start_fn_t)(
+    size_t count,
+    struct ompi_request_t ** requests
+);
+
 /*
  * Required function to free the request and any associated resources.
  */
@@ -105,10 +130,11 @@ struct ompi_request_t {
     opal_free_list_item_t super;                /**< Base type */
     ompi_request_type_t req_type;               /**< Enum indicating the type of the request */
     ompi_status_public_t req_status;            /**< Completion status */
-    volatile void *req_complete;                /**< Flag indicating wether request has completed */
+    volatile void *req_complete;                /**< Flag indicating whether request has completed */
     volatile ompi_request_state_t req_state;    /**< enum indicate state of the request */
     bool req_persistent;                        /**< flag indicating if the this is a persistent request */
     int req_f_to_c_index;                       /**< Index in Fortran <-> C translation array */
+    ompi_request_start_fn_t req_start;          /**< Called by MPI_START and MPI_STARTALL */
     ompi_request_free_fn_t req_free;            /**< Called by free */
     ompi_request_cancel_fn_t req_cancel;        /**< Optional function to cancel the request */
     ompi_request_complete_fn_t req_complete_cb; /**< Called when the request is MPI completed */
@@ -123,11 +149,14 @@ typedef struct ompi_request_t ompi_request_t;
 
 
 /**
- * Padded struct to maintain back compatibiltiy.
+ * Padded struct to maintain back compatibility.
  * See ompi/communicator/communicator.h comments with struct ompi_communicator_t
  * for full explanation why we chose the following padding construct for predefines.
  */
-#define PREDEFINED_REQUEST_PAD (sizeof(void*) * 32)
+#define PREDEFINED_REQUEST_PAD 256
+
+#define REQUEST_PENDING        (void *)0L
+#define REQUEST_COMPLETED      (void *)1L
 
 struct ompi_predefined_request_t {
     struct ompi_request_t request;
@@ -167,7 +196,7 @@ typedef struct ompi_predefined_request_t ompi_predefined_request_t;
  * This function should be called only from the MPI layer. It should
  * never be called from the PML. It take care of the upper level clean-up.
  * When the user call MPI_Request_free we should release all MPI level
- * ressources, so we have to call this function too.
+ * resources, so we have to call this function too.
  */
 #define OMPI_REQUEST_FINI(request)                                      \
 do {                                                                    \
@@ -179,6 +208,28 @@ do {                                                                    \
     }                                                                   \
 } while (0);
 
+/*
+ * Except in procedures that return MPI_ERR_IN_STATUS, the MPI_ERROR
+ * field of a status object shall never be modified
+ * See MPI-1.1 doc, sec 3.2.5, p.22
+ *
+ * Add a small macro that helps setting the status appropriately
+ * depending on the use case
+ */
+#define OMPI_COPY_STATUS(pdst, src, is_err_in_status)                   \
+do {                                                                    \
+    if (is_err_in_status) {                                             \
+        *(pdst) = (src);                                                \
+    }                                                                   \
+    else {                                                              \
+        (pdst)->MPI_TAG = (src).MPI_TAG;                                \
+        (pdst)->MPI_SOURCE = (src).MPI_SOURCE;                          \
+        (pdst)->_ucount = (src)._ucount;                                \
+        (pdst)->_cancelled = (src)._cancelled;                          \
+    }                                                                   \
+} while(0);
+
+
 /**
  * Non-blocking test for request completion.
  *
@@ -187,7 +238,7 @@ do {                                                                    \
  * @param status (OUT)   Status of completed request.
  * @return               OMPI_SUCCESS or failure status.
  *
- * Note that upon completion, the request is freed, and the
+ * Note that upon completion, the request completed without error is freed, and the
  * request handle at index set to NULL.
  */
 typedef int (*ompi_request_test_fn_t)(ompi_request_t ** rptr,
@@ -203,7 +254,7 @@ typedef int (*ompi_request_test_fn_t)(ompi_request_t ** rptr,
  * @param status (OUT)   Status of completed request.
  * @return               OMPI_SUCCESS or failure status.
  *
- * Note that upon completion, the request is freed, and the
+ * Note that upon completion, the request completed without error is freed, and the
  * request handle at index set to NULL.
  */
 typedef int (*ompi_request_test_any_fn_t)(size_t count,
@@ -216,14 +267,14 @@ typedef int (*ompi_request_test_any_fn_t)(size_t count,
  *
  * @param count (IN)      Number of requests
  * @param requests (IN)   Array of requests
- * @param completed (OUT) Flag indicating wether all requests completed.
+ * @param completed (OUT) Flag indicating whether all requests completed.
  * @param statuses (OUT)  Array of completion statuses.
  * @return                OMPI_SUCCESS or failure status.
  *
- * This routine returns completed==true if all requests have completed.
- * The statuses parameter is only updated if all requests completed. Likewise,
- * the requests array is not modified (no requests freed), unless all requests
- * have completed.
+ * This routine returns completed==true if all requests completed without errors
+ * have completed. The statuses parameter is only updated if all requests completed.
+ * Likewise, the requests array is not modified (no requests freed), unless all
+ * requests have completed.
  */
 typedef int (*ompi_request_test_all_fn_t)(size_t count,
                                           ompi_request_t ** requests,
@@ -246,7 +297,10 @@ typedef int (*ompi_request_test_some_fn_t)(size_t count,
                                            int * indices,
                                            ompi_status_public_t * statuses);
 /**
- * Wait (blocking-mode) for one requests to complete.
+ * Wait (blocking-mode) for one requests to complete. This function is slightly
+ * different from the MPI counter-part as it does not release the requests
+ * completed with error. Instead, the caller is responsible to call the
+ * ompi_request_free.
  *
  * @param request (IN)    Pointer to request.
  * @param status (OUT)    Status of completed request.
@@ -256,7 +310,10 @@ typedef int (*ompi_request_test_some_fn_t)(size_t count,
 typedef int (*ompi_request_wait_fn_t)(ompi_request_t ** req_ptr,
                                       ompi_status_public_t * status);
 /**
- * Wait (blocking-mode) for one of N requests to complete.
+ * Wait (blocking-mode) for one of N requests to complete. This function is
+ * slightly different from the MPI counter-part as it does not release the
+ * requests completed with error. Instead, the caller is responsible to call
+ * the ompi_request_free.
  *
  * @param count (IN)      Number of requests
  * @param requests (IN)   Array of requests
@@ -270,7 +327,10 @@ typedef int (*ompi_request_wait_any_fn_t)(size_t count,
                                           int *index,
                                           ompi_status_public_t * status);
 /**
- * Wait (blocking-mode) for all of N requests to complete.
+ * Wait (blocking-mode) for all of N requests to complete. This function is
+ * slightly different from the MPI counter-part as it does not release the
+ * requests completed with error. Instead, the caller is responsible to call
+ * the ompi_request_free.
  *
  * @param count (IN)      Number of requests
  * @param requests (IN)   Array of requests
@@ -282,7 +342,10 @@ typedef int (*ompi_request_wait_all_fn_t)(size_t count,
                                           ompi_request_t ** requests,
                                           ompi_status_public_t * statuses);
 /**
- * Wait (blocking-mode) for some of N requests to complete.
+ * Wait (blocking-mode) for some of N requests to complete. This function is
+ * slightly different from the MPI counter-part as it does not release the
+ * requests completed with error. Instead, the caller is responsible to call
+ * the ompi_request_free.
  *
  * @param count (IN)        Number of requests
  * @param requests (INOUT)  Array of requests
@@ -316,17 +379,14 @@ typedef struct ompi_request_fns_t {
  * Globals used for tracking requests and request completion.
  */
 OMPI_DECLSPEC extern opal_pointer_array_t   ompi_request_f_to_c_table;
-OMPI_DECLSPEC extern size_t                 ompi_request_waiting;
-OMPI_DECLSPEC extern size_t                 ompi_request_completed;
-OMPI_DECLSPEC extern size_t                 ompi_request_failed;
-OMPI_DECLSPEC extern int32_t                ompi_request_poll;
-OMPI_DECLSPEC extern opal_recursive_mutex_t ompi_request_lock;
-OMPI_DECLSPEC extern opal_condition_t       ompi_request_cond;
 OMPI_DECLSPEC extern ompi_predefined_request_t        ompi_request_null;
 OMPI_DECLSPEC extern ompi_predefined_request_t        *ompi_request_null_addr;
 OMPI_DECLSPEC extern ompi_request_t         ompi_request_empty;
 OMPI_DECLSPEC extern ompi_status_public_t   ompi_status_empty;
 OMPI_DECLSPEC extern ompi_request_fns_t     ompi_request_functions;
+#if MPI_VERSION >= 4
+OMPI_DECLSPEC extern ompi_request_t         ompi_request_empty_send;
+#endif
 
 /**
  * Initialize the MPI_Request subsystem; invoked during MPI_INIT.
@@ -334,15 +394,9 @@ OMPI_DECLSPEC extern ompi_request_fns_t     ompi_request_functions;
 int ompi_request_init(void);
 
 /**
- * Free a persistent request to a MPI_PROC_NULL peer (there's no
- * freelist to put it back to, so we have to actually OBJ_RELEASE it).
+ * Create a persistent request that does nothing (e.g., to MPI_PROC_NULL).
  */
-OMPI_DECLSPEC int ompi_request_persistent_proc_null_free(ompi_request_t **request);
-
-/**
- * Shut down the MPI_Request subsystem; invoked during MPI_FINALIZE.
- */
-int ompi_request_finalize(void);
+int ompi_request_persistent_noop_create(ompi_request_t **request);
 
 /**
  * Cancel a pending request.
@@ -374,28 +428,75 @@ static inline int ompi_request_free(ompi_request_t** request)
 #define ompi_request_wait_all   (ompi_request_functions.req_wait_all)
 #define ompi_request_wait_some  (ompi_request_functions.req_wait_some)
 
+#if OPAL_ENABLE_FT_MPI
+OMPI_DECLSPEC bool ompi_request_is_failed_fn(ompi_request_t *req);
+#define ompi_request_is_failed(req) OPAL_UNLIKELY(ompi_ftmpi_enabled? ompi_request_is_failed_fn(req): false)
+
+#include "ompi/mca/coll/base/coll_tags.h"
+
+static inline bool ompi_request_tag_is_ft(int tag) {
+    return (tag <= MCA_COLL_BASE_TAG_FT_BASE && tag >= MCA_COLL_BASE_TAG_FT_END);
+}
+
+static inline bool ompi_request_tag_is_collective(int tag) {
+    return ((tag <= MCA_COLL_BASE_TAG_BASE && tag >= MCA_COLL_BASE_TAG_END) && !ompi_request_tag_is_ft(tag));
+}
+#endif /* OPAL_ENABLE_FT_MPI */
+
 /**
  * Wait a particular request for completion
  */
 
 static inline void ompi_request_wait_completion(ompi_request_t *req)
 {
-    if (opal_using_threads () && !REQUEST_COMPLETE(req)) {
-        ompi_wait_sync_t sync;
-        WAIT_SYNC_INIT(&sync, 1);
+    if (opal_using_threads ()) {
+        if(!REQUEST_COMPLETE(req)) {
+            void *_tmp_ptr;
+            ompi_wait_sync_t sync;
 
-        if (OPAL_ATOMIC_CMPSET_PTR(&req->req_complete, REQUEST_PENDING, &sync)) {
-            SYNC_WAIT(&sync);
-        } else {
-            /* completed before we had a chance to swap in the sync object */
-            WAIT_SYNC_SIGNALLED(&sync);
-        }
 
-        assert(REQUEST_COMPLETE(req));
-        WAIT_SYNC_RELEASE(&sync);
+#if OPAL_ENABLE_FT_MPI
+    redo:
+            if(OPAL_UNLIKELY( ompi_request_is_failed(req) )) {
+                return;
+            }
+#endif /* OPAL_ENABLE_FT_MPI */
+            _tmp_ptr = REQUEST_PENDING;
+
+            WAIT_SYNC_INIT(&sync, 1);
+
+            if (OPAL_ATOMIC_COMPARE_EXCHANGE_STRONG_PTR(&req->req_complete, &_tmp_ptr, &sync)) {
+                SYNC_WAIT(&sync);
+            } else {
+                /* completed before we had a chance to swap in the sync object */
+                WAIT_SYNC_SIGNALLED(&sync);
+            }
+
+#if OPAL_ENABLE_FT_MPI
+            if (OPAL_UNLIKELY(OMPI_SUCCESS != sync.status)) {
+                OPAL_OUTPUT_VERBOSE((50, ompi_ftmpi_output_handle, "Status %d reported for sync %p rearming req %p", sync.status, (void*)&sync, (void*)req));
+                _tmp_ptr = &sync;
+                if (OPAL_ATOMIC_COMPARE_EXCHANGE_STRONG_PTR(&req->req_complete, &_tmp_ptr, REQUEST_PENDING)) {
+                    opal_output_verbose(10, ompi_ftmpi_output_handle, "Status %d reported for sync %p rearmed req %p", sync.status, (void*)&sync, (void*)req);
+                    WAIT_SYNC_RELEASE(&sync);
+                    goto redo;
+                }
+            }
+#endif /* OPAL_ENABLE_FT_MPI */
+            assert(REQUEST_COMPLETE(req));
+            WAIT_SYNC_RELEASE(&sync);
+     }
+     opal_atomic_rmb();
     } else {
         while(!REQUEST_COMPLETE(req)) {
             opal_progress();
+#if OPAL_ENABLE_FT_MPI
+            /* Check to make sure that process failure did not break the
+             * request. */
+            if(OPAL_UNLIKELY( ompi_request_is_failed(req) )) {
+                break;
+            }
+#endif /* OPAL_ENABLE_FT_MPI */
         }
     }
 }
@@ -417,31 +518,44 @@ static inline int ompi_request_complete(ompi_request_t* request, bool with_signa
 {
     int rc = 0;
 
-    if( NULL != request->req_complete_cb) {
-        rc = request->req_complete_cb( request );
+    if(NULL != request->req_complete_cb) {
+        /* Set the request cb to NULL to allow resetting in the callback */
+        ompi_request_complete_fn_t fct = request->req_complete_cb;
         request->req_complete_cb = NULL;
+        rc = fct( request );
     }
 
     if (0 == rc) {
-        if( OPAL_LIKELY(with_signal) ) {
-            if(!OPAL_ATOMIC_CMPSET_PTR(&request->req_complete, REQUEST_PENDING, REQUEST_COMPLETED)) {
-                ompi_wait_sync_t *tmp_sync = (ompi_wait_sync_t *) OPAL_ATOMIC_SWAP_PTR(&request->req_complete,
-                                                                                       REQUEST_COMPLETED);
-                /* In the case where another thread concurrently changed the request to REQUEST_PENDING */
-                if( REQUEST_PENDING != tmp_sync )
-                    wait_sync_update(tmp_sync, 1, request->req_status.MPI_ERROR);
-            }
-        } else
-            request->req_complete = REQUEST_COMPLETED;
+        if (OPAL_LIKELY(with_signal)) {
 
-        if( OPAL_UNLIKELY(MPI_SUCCESS != request->req_status.MPI_ERROR) ) {
-            ompi_request_failed++;
+            ompi_wait_sync_t *tmp_sync = (ompi_wait_sync_t *) OPAL_ATOMIC_SWAP_PTR(&request->req_complete,
+                                                                                    REQUEST_COMPLETED);
+            if( REQUEST_PENDING != tmp_sync ) {
+                wait_sync_update(tmp_sync, 1, request->req_status.MPI_ERROR);
+            }
+        } else {
+            request->req_complete = REQUEST_COMPLETED;
         }
     }
 
     return OMPI_SUCCESS;
 }
 
+static inline int ompi_request_set_callback(ompi_request_t* request,
+                                            ompi_request_complete_fn_t cb,
+                                            void* cb_data)
+{
+    request->req_complete_cb_data = cb_data;
+    request->req_complete_cb = cb;
+    /* If request is completed and the callback is not called, need to call callback */
+    if ((NULL != request->req_complete_cb) && (request->req_complete == REQUEST_COMPLETED)) {
+        ompi_request_complete_fn_t fct = request->req_complete_cb;
+        request->req_complete_cb = NULL;
+        return fct( request );
+    }
+    return OMPI_SUCCESS;
+}
+
 END_C_DECLS
 
 #endif
